﻿Imports Inventor
Imports System
Imports System.Windows.Forms

Public Class Clipboard_Palette
    ' Constants for Windows API calls

    Private Const WM_DRAWCLIPBOARD As Integer = &H308
    Private Const WM_CHANGECBCHAIN As Integer = &H30D

    Private oInvApp As Inventor.Application
    Private _nxtCbVwrHWnd As IntPtr

    ' Boolean to control access to clipboard data
    Private _internalHold As Boolean = False

    ' Counter for our visible clipboard name
    Private _clipboardCounter As Integer = 0

    Declare Auto Function SetClipboardViewer Lib "user32" (ByVal HWnd As IntPtr) As IntPtr
    Declare Auto Function ChangeClipboardChain Lib "user32" (ByVal hWndRemove As IntPtr, ByVal hWndNewNext As IntPtr) As Boolean

    Declare Auto Function SendMessage Lib "User32" (ByVal HWnd As IntPtr, ByVal Msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr) As Long

    Dim oEachObj As Object
    Dim oSelSet As SelectSet
    Dim oTextBox As Inventor.TextBox
    Dim i As Integer

    Private WithEvents m_InteractionEvents As InteractionEvents
    Private WithEvents m_MouseEvents As MouseEvents
    Private m_flagMouseDown As Boolean
    Private m_MouseStartPt As Inventor.Point2d
    Private m_StartModelPt As Inventor.Point
    Private oPasteTextBox As Inventor.TextBox

    Public Sub New()

        ' This call is required by the Windows Form Designer

        InitializeComponent()

        ' Register ourselves to handle clipboard modifications

        _nxtCbVwrHWnd = SetClipboardViewer(Handle)

    End Sub

    Public Sub setInvApp(ByVal oApp As Inventor.Application)
        oInvApp = oApp
    End Sub

    Protected Overrides Sub WndProc(ByRef m As Message)
        Debug.Print(m.Msg.ToString)
        Select Case m.Msg

            ' The clipboard has changed          

            Case Is = WM_DRAWCLIPBOARD
                Debug.Print("***********")
                'Dim oo As String = oInvApp.UserName
                If Not oInvApp Is Nothing Then
                    If TypeOf oInvApp.ActiveDocument Is DrawingDocument And TypeOf oInvApp.ActiveEditObject Is Sketch Then
                        Dim oSketch As DrawingSketch = oInvApp.ActiveEditObject
                        Dim oActiveDoc As DrawingDocument = oInvApp.ActiveDocument

                        Dim oEachObj As Object
                        oSelSet = oInvApp.ActiveDocument.SelectSet

                        For Each oEachObj In oSelSet
                            'If TypeOf (oEachObj) Is SketchEntity Then
                            If TypeOf (oEachObj) Is Inventor.TextBox Then
                                oTextBox = oEachObj
                                If TypeOf oSketch.Parent Is TitleBlockDefinition Then
                                    Dim oTitleBlkDef As TitleBlockDefinition = oSketch.Parent
                                    CopyTextBox(oTextBox, oTitleBlkDef.Name)
                                Else
                                    CopyTextBox(oTextBox, "N/A")
                                End If

                            End If
                            'End If
                        Next

                    End If
                End If

                SendMessage(_nxtCbVwrHWnd, m.Msg, m.WParam, m.LParam)

                ' Another clipboard viewer has removed itself

            Case Is = WM_CHANGECBCHAIN

                If m.WParam = CType(_nxtCbVwrHWnd, IntPtr) Then
                    _nxtCbVwrHWnd = m.LParam
                Else
                    SendMessage(_nxtCbVwrHWnd, m.Msg, m.WParam, m.LParam)
                End If

        End Select

        MyBase.WndProc(m)
    End Sub


    Private Sub CopyTextBox(ByVal oTextBox As Inventor.TextBox, ByVal oTitleBlockName As String)

        Dim oClsTextBoxInfo As clsTextBoxInfo = New clsTextBoxInfo


        oClsTextBoxInfo.oFormattedText = oTextBox.FormattedText
        oClsTextBoxInfo.oHeight = oTextBox.Height
        oClsTextBoxInfo.oLayer = oTextBox.Layer
        oClsTextBoxInfo.oRotation = oTextBox.Rotation
        oClsTextBoxInfo.oStyle = oTextBox.Style
        oClsTextBoxInfo.oWidthScale = oTextBox.Width
        oClsTextBoxInfo.oVerJust = oTextBox.VerticalJustification
        oClsTextBoxInfo.oHorJust = oTextBox.HorizontalJustification
        oClsTextBoxInfo.oTitleBlockName = oTitleBlockName
        oClsTextBoxInfo.oTextBoxName = oTextBox.Text

        AddDataToGrid(oClsTextBoxInfo)


    End Sub

    Private Sub AddDataToGrid(ByVal oClsTextBoxInfo As clsTextBoxInfo)

        Dim newRow As New DataGridViewRow()
        newRow.Tag = oClsTextBoxInfo

        ' Increment our counter
            _clipboardCounter += 1

            ' Create and add a cell for the name, using our counter

            Dim newNameCell As New DataGridViewTextBoxCell
            newNameCell.Value = "Clipboard " & _clipboardCounter
            newRow.Cells.Add(newNameCell)

            ' Get the current time and place that in another cell

            Dim newTimeCell As New DataGridViewTextBoxCell
            newTimeCell.Value = Now.ToLongTimeString
        newRow.Cells.Add(newTimeCell)

        Dim newTitleBlockCell As New DataGridViewTextBoxCell
        newTitleBlockCell.Value = oClsTextBoxInfo.oTitleBlockName
        newRow.Cells.Add(newTitleBlockCell)

        Dim newTextBoxNameCell As New DataGridViewTextBoxCell
        newTextBoxNameCell.Value = oClsTextBoxInfo.oTextBoxName
        newRow.Cells.Add(newTextBoxNameCell)



            ' Add our row to the data grid and select it
            Try
                clipboardDataGridView.Rows.Add(newRow)
                clipboardDataGridView.FirstDisplayedScrollingRowIndex = _
                  clipboardDataGridView.Rows.Count - 1
                newRow.Selected = True
            Catch ex As Exception
                MsgBox(ex.ToString())
        End Try
 

    End Sub
    Private Sub RemoveAllToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RemoveAllToolStripMenuItem.Click
        clipboardDataGridView.Rows.Clear()
    End Sub

    Private Sub PasteToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PasteToolStripMenuItem.Click
        If clipboardDataGridView.SelectedRows.Count = 1 Then
            myPaste(clipboardDataGridView.SelectedRows.Item(0).Tag)
        End If
    End Sub

    Private Sub myPaste(ByVal oClsTextBoxInfo As clsTextBoxInfo)

        If TypeOf oInvApp.ActiveDocument Is DrawingDocument And TypeOf oInvApp.ActiveEditObject Is Sketch Then

            Dim oSketch As Sketch = oInvApp.ActiveEditObject
            oPasteTextBox = oSketch.TextBoxes.AddFitted(oInvApp.TransientGeometry.CreatePoint2d(0, 0), oClsTextBoxInfo.oFormattedText, oClsTextBoxInfo.oStyle)

            If m_InteractionEvents Is Nothing Then
                m_InteractionEvents = oInvApp.CommandManager.CreateInteractionEvents()
            Else
                m_InteractionEvents.Stop()
            End If

            m_InteractionEvents.InteractionDisabled = False

            If m_MouseEvents Is Nothing Then
                m_MouseEvents = m_InteractionEvents.MouseEvents
                m_MouseEvents.MouseMoveEnabled = True
            End If

            m_InteractionEvents.Name = "MyPaste"

            m_MouseStartPt = oInvApp.TransientGeometry.CreatePoint2d(0, 0)

            m_flagMouseDown = False
            m_InteractionEvents.Start()


        End If
    End Sub

    Private Sub m_InteractionEvents_OnTerminate() Handles m_InteractionEvents.OnTerminate
        m_InteractionEvents.InteractionGraphics.PreviewClientGraphics.Delete()
        m_flagMouseDown = False
        m_InteractionEvents.Stop()
        m_MouseEvents = Nothing
        m_InteractionEvents = Nothing
    End Sub

    Private Sub m_MouseEvents_OnMouseClick(ByVal Button As Inventor.MouseButtonEnum, ByVal ShiftKeys As Inventor.ShiftStateEnum, ByVal ModelPosition As Inventor.Point, ByVal ViewPosition As Inventor.Point2d, ByVal View As Inventor.View) Handles m_MouseEvents.OnMouseClick
        If m_InteractionEvents.Name = "MyPaste" Then

            If Not oPasteTextBox Is Nothing Then
                Dim oPt2d As Point2d = oInvApp.TransientGeometry.CreatePoint2d(ModelPosition.X, ModelPosition.Y)
                oPasteTextBox.Origin = oPt2d
            End If

            m_flagMouseDown = False
            m_InteractionEvents.Stop()
            m_MouseEvents = Nothing
            m_InteractionEvents = Nothing
        End If
    End Sub

    Private Sub m_MouseEvents_OnMouseMove(ByVal Button As Inventor.MouseButtonEnum, ByVal ShiftKeys As Inventor.ShiftStateEnum, ByVal ModelPosition As Inventor.Point, ByVal ViewPosition As Inventor.Point2d, ByVal View As Inventor.View) Handles m_MouseEvents.OnMouseMove
        If m_InteractionEvents.Name = "MyPaste" Then
            Dim oPt2d As Point2d = oInvApp.TransientGeometry.CreatePoint2d(ModelPosition.X, ModelPosition.Y)
            If Not oPasteTextBox Is Nothing Then
                oPasteTextBox.Origin = oPt2d
            End If
        End If
    End Sub
End Class


Class clsTextBoxInfo

    Public oFormattedText As String
    Public oHeight As Double
    Public oLayer As Layer
    Public oRotation As Double
    Public oStyle As TextStyle
    Public oWidthScale As Double
    Public oVerJust As VerticalTextAlignmentEnum
    Public oHorJust As HorizontalTextAlignmentEnum

    Public oTitleBlockName As String
    Public oTextBoxName As String

End Class
